; ==================================================================
; HSFinaleNav: Autohotkey script to make Finale's bookmarks
; selectable via button clicks in an always-on-top window.
;
; Autohotkey from https://www.autohotkey.com/ must be installed!
;
; First, create you bookmarks ("Positionsmarken" in the German version)
; then launch the script. A modal window will pop up to show all
; bookmarks as buttons. Clicking one will select that bookmark within
; Finale. If you create or delete bookmarks, re-launch the script.
;
; Finale always sorts its bookmarks, the script shows the same order.
; Version 2.0 on 1.1.2019 by Harald Schollmeyer, hschollm@gmx.net
; ==================================================================

; Changes and Bugfixes since Version 1.0:
; - F12 invokes the script, but only if Finale is active
; - the script searches the View Menu on startup and every button click
; - buttons go in columns and rows and resize themselves according window size



#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
SendMode Input  ; Recommended for new scripts due to its superior speed and reliability.
SetWorkingDir %A_ScriptDir%  ; Ensures a consistent starting directory.
#SingleInstance force


#IfWinActive ahk_class Finale
F12::


; ------------------------------------------------------------------------
; Set up and initalize new GUI Window
; ------------------------------------------------------------------------

; the new GUI window should always be topmost:
Gui, +AlwaysOnTop +Resize -MaximizeBox -MinimizeBox

; get the Finale window and its main menu handle
hWnd := WinExist("ahk_class Finale")
hMenu := DllCall("GetMenu", "UInt", hWnd)

; search the View menu, get its handle
ViewMenuPosition := SearchViewMenu(hMenu)
ViewMenu := DllCall("GetSubMenu", "UInt", hMenu, "Int", ViewMenuPosition)

; get the  bookmarks submenu, 7th menu counted 0-based
BookmarksMenu := DllCall("GetSubMenu", "UInt", ViewMenu, "Int", 6)

; to get the NumberOfBookmarks, get the "Bookmarks" menu size minus the
; two leading function items and the one delimiter line, thus making -3 items
NumberOfBookmarks := DllCall("GetMenuItemCount", "UInt", BookmarksMenu) - 3

if (NumberOfBookmarks < 1)
{
	MsgBox No Bookmarks found!
	ExitApp
}

; set up a GUI button for each bookmark in a loop
Loop %NumberOfBookmarks%
{
   ; initalize huge variable for the bookmark's name
   VarSetCapacity(BookmarkName, 1024, 0)
   DllCall("User32.dll\GetMenuString", "Ptr", BookmarksMenu, "UInt", A_Index - 1 + 3
   , "Str", BookmarkName, "Int", 512 , "UInt", 0x0400, "Int")

   ; create a new button with the bookmark's name, but all share one common event handler
   Gui, Add, Button, gButtonHandler, %BookmarkName%
}

; when all buttons are created show the GUI
Gui,Show, x40 y 40 w 300 h 300, HSFinaleNav

return

; ------------------------------------------------------------------------
; GUI Resize Handler: re-calculate columns and rows of all buttons
; ------------------------------------------------------------------------

GuiSize:
Columns := Floor( A_GuiWidth / 100)
Rows := Floor( (NumberOfBookmarks+2) / Columns )
ButtonWidth := A_GuiWidth / Columns
ButtonHeigth := A_GuiHeight / Rows

Loop %NumberOfBookmarks%
{
	Column := Mod( (A_Index-1) , Columns)
	Row := Floor((A_Index-1) / Columns)
	x := Column * ButtonWidth
	y := Row * ButtonHeigth
	b := "Button" . A_Index
	ControlMove, %b%, x+10 , y+30, ButtonWidth-5, ButtonHeigth-5
}
return


; ------------------------------------------------------------------------
; The one event handler for all buttons
; ------------------------------------------------------------------------
ButtonHandler:

   ; which button was clicked? The one which has focus:
   GuiControlGet, b, Focus

   ; extract its number from its auto-created name "Button1","Button2"...
   StringRight, c, b, StrLen(b)-StrLen("Button")

   ; calculate the menu item to be clicked: menu starts with two functions,
   ; then a delimiter line, making a +3 necessary
   clicks := c + 3

   ; execute the click in the View Menu, 7th submenu as above, but now 1-based
   IncrementViewMenuPosition := SearchViewMenu(global hMenu) + 1
   WinMenuSelectItem, ahk_class Finale,, %IncrementViewMenuPosition%&, 7&, %clicks%&

   ; return the focus to Finale:
   WinActivate, ahk_class Finale

return


; ------------------------------------------------------------------------
; Determine the View menu's position: Check whether the first submenu is a
; window menu (menu string empty means: document maximized) or  contains
; text (menu string len>0 means it's the "File" menu). Accordingly, the
; View menu is either at position 3 or 4
; ------------------------------------------------------------------------
SearchViewMenu(ByRef menu)
{
	r := 4
	VarSetCapacity(result, 1024, 0)
	DllCall("GetMenuString", "Ptr", menu, "UInt", 0
	   , "Str", result, "Int", 512 , "UInt", 0x0400, "Int")
	if (StrLen(result) > 0)
		r := 3
	return r
}

GuiClose:
Gui, Destroy


#IfWinActive

